@@ -19,12 +19,12 @@ from wheatley.arg_parsing import parse_peal_speed, PealSpeedParseError, parse_ca
 from wheatley.row_generation import RowGenerator, ComplibCompositionGenerator
 from wheatley.row_generation import MethodPlaceNotationGenerator
 from wheatley.row_generation.complib_composition_generator import PrivateCompError, InvalidCompError
-from wheatley.row_generation.method_place_notation_generator import MethodNotFoundError, generator_from_special_title
+from wheatley.row_generation.method_place_notation_generator import MethodNotFoundError, \
+                                                                    generator_from_special_title
 
 
-def row_generator(args):
+def create_row_generator(args):
     """ Generates a row generator according to the given CLI arguments. """
-
     if "comp" in args and args.comp is not None:
         try:
             row_gen = ComplibCompositionGenerator(args.comp)
@@ -52,7 +52,7 @@ def row_generator(args):
     return row_gen
 
 
-def rhythm(args):
+def create_rhythm(args):
     """ Generates a rhythm object according to the given CLI arguments. """
 
     try:
@@ -60,11 +60,21 @@ def rhythm(args):
     except PealSpeedParseError as e:
         sys.exit(str(e))
 
+    # Sets the minimum number of bells that Wheatley will use in order to deduce a rhythm.  Setting this to
+    # larger numbers will make Wheatley more stable during the pull-off, whereas smaller numbers will make
+    # Wheatley more sensitive to user's pull-off speed.  If this is larger than `--max_bells_in_dataset`,
+    # this will be clamped to fit.
+    min_bells_in_dataset = 4
+
+    # Clamp min_bells_in_dataset to not be bigger than max_bells_in_dataset
+    min_bells_in_dataset = min(min_bells_in_dataset, args.max_bells_in_dataset)
+
     regression = RegressionRhythm(
         args.inertia,
         handstroke_gap=args.handstroke_gap,
         peal_speed=peal_speed,
-        max_rows_in_dataset=args.max_rows_in_dataset
+        min_bells_in_dataset=min_bells_in_dataset,
+        max_bells_in_dataset=args.max_bells_in_dataset
     )
 
     if not args.keep_going:
@@ -84,11 +94,15 @@ def configure_logging():
     logging.getLogger(WaitForUserRhythm.logger_name).setLevel(logging.INFO)
 
 
-def main():
+def main(override_args=None, stop_on_join_tower=False):
     """
     The main function of the bot.
     This parses the CLI arguments, creates the Rhythm, RowGenerator and Bot objects, then starts
     the bot's mainloop.
+
+    The two optional arguments are used by the integration tester to give Wheatley artificial argument values
+    (override_args) and make Wheatley exit with error code 0 on joining a tower so that hanging forever
+    can be differentiated from Wheatley's normal behaviour of sitting in an infinite loop waiting for input.
     """
 
     # Try to read the file with the version number, if not print an error and set a poison value
@@ -101,111 +115,51 @@ def main():
     except IOError:
         __version__ = "<NO VERSION FILE FOUND>"
 
-    # Parse the arguments
+    # PARSE THE ARGUMENTS
+
     parser = argparse.ArgumentParser(
         description="A bot to fill in bells during ringingroom.com practices"
     )
 
-    parser.add_argument(
+    # Tower arguments
+    tower_group = parser.add_argument_group("Tower arguments")
+    tower_group.add_argument(
         "room_id",
         type=int,
         help="The numerical ID of the tower to join, represented as a row on 9 bells, \
               e.g. 763451928."
     )
-    parser.add_argument(
+    tower_group.add_argument(
         "--url",
         default="https://ringingroom.com",
         type=str,
         help="The URL of the server to join (defaults to 'https://ringingroom.com')"
     )
-    parser.add_argument(
-        "-u", "--use-up-down-in",
-        action="store_true",
-        help="If set, then the bot will automatically go into changes after two rounds have been \
-              rung."
-    )
-    parser.add_argument(
-        "-s", "--stop-at-rounds",
-        action="store_true",
-        help="If set, then the bot will stand its bells after rounds is reached."
-    )
-    parser.add_argument(
-        "-H", "--handbell-style",
-        action="store_true",
-        help="If set, then the bot will ring 'handbell style', i.e. ringing two strokes of \
-              rounds then straight into changes, and stopping at the first set of rounds. By \
-              default, it will ring 'towerbell style', i.e. only taking instructions from the \
-              ringing-room calls. This is equivalent to using the '-us' flags."
-    )
-    parser.add_argument(
-        "--version",
-        action="version",
-        version=f"Wheatley v{__version__}"
+    tower_group.add_argument(
+        "--name",
+        default=None,
+        type=str,
+        help="If set, then the bot will ring bells assigned to the given name. \
+             When not set, the bot rings unassigned bells."
     )
 
-    # Rhythm arguments
-    parser.add_argument(
-        "-k", "--keep-going",
-        action="store_true",
-        help="If set, the bot will not wait for users to ring - instead, it will push on with the \
-              rhythm."
-    )
-    parser.add_argument(
-        "-w", "--wait",
-        action="store_true",
-        help="Legacy parameter, which is now set by default. The previous default behaviour of not waiting \
-              can be set with '-k'/'--keep-going'."
-    )
-    parser.add_argument(
-        "-I", "--inertia",
-        type=float,
-        default=0.5,
-        help="Overrides the bot's 'inertia' - now much the bot will take other ringers' positions \
-              into account when deciding when to ring.  0.0 means it will cling as closely as \
-              possible to the current rhythm, 1.0 means that it will completely ignore the other \
-              ringers. By default, it will set a value depending on what proportion of the bells \
-              are user controlled."
-    )
-    parser.add_argument(
-        "-S", "--peal-speed",
-        default="2h58",
-        help="Sets the default speed that the bot will ring (assuming a peal of 5040 changes), \
-              though this will usually be adjusted by the bot whilst ringing to keep with other \
-              ringers.  Example formatting: '3h4' = '3h4m' = '3h04m' = '3h04' = '184m' = '184'. \
-              Defaults to '2h58'."
-    )
-    parser.add_argument(
-        "-G", "--handstroke-gap",
-        type=float,
-        default=1.0,
-        help="Sets the handstroke gap as a factor of the space between two bells.  Defaults to \
-              '1.0'."
-    )
-    parser.add_argument(
-        "-X", "--max-rows-in-dataset",
-        type=float,
-        default=3.0,
-        help="Sets the maximum number of rows that Wheatley will store to determine the current \
-              ringing speed.  If you make this larger, then will be more consistent but less \
-              quick to respond to changes in rhythm.  Defaults to '3.0'.  Setting both this and \
-              --inertia to a very small values could result in Wheatley ringing ridiculously \
-              quickly."
-    )
+    # Row generation arguments
+    row_gen_group = parser.add_argument_group("Row generation arguments")
 
-    # Row generator arguments
-    row_gen_group = parser.add_mutually_exclusive_group(required=True)
-    row_gen_group.add_argument(
+    # An mutual exclusion group to disallow specifying both a method *and* a CompLib comp
+    comp_method_group = row_gen_group.add_mutually_exclusive_group(required=True)
+    comp_method_group.add_argument(
         "-c", "--comp",
         type=int,
         help="The ID of the complib composition you want to ring"
     )
-    row_gen_group.add_argument(
+    comp_method_group.add_argument(
         "-m", "--method",
         type=str,
         help="The title of the method you want to ring"
     )
 
-    parser.add_argument(
+    row_gen_group.add_argument(
         "-b",
         "--bob",
         default="14",
@@ -216,7 +170,7 @@ def main():
               "20:7/0:4" => a 70 bob 20 changes into a lead and a 14 bob at the lead end. \
               "3: 5/9: 5" => bobs in Stedman Triples.  Defaults to "14".'
     )
-    parser.add_argument(
+    row_gen_group.add_argument(
         "-n",
         "--single",
         default="1234",
@@ -226,9 +180,88 @@ def main():
               "20: 7890" => a 7890 single taking effect 20 changes into a lead (the Half Lead for \
               Surprise Royal). "3: 567/9: 567" => singles in Stedman Triples.  Defaults to "1234".'
     )
+    row_gen_group.add_argument(
+        "-u", "--use-up-down-in",
+        action="store_true",
+        help="If set, then the Wheatley will automatically go into changes after two rounds have been \
+              rung."
+    )
+    row_gen_group.add_argument(
+        "-s", "--stop-at-rounds",
+        action="store_true",
+        help="If set, then Wheatley will stand his bells the first time rounds is reached."
+    )
+    row_gen_group.add_argument(
+        "-H", "--handbell-style",
+        action="store_true",
+        help="If set, then Wheatley will ring 'handbell style', i.e. ringing two strokes of \
+              rounds then straight into changes, and stopping at the first set of rounds. By \
+              default, he will ring 'towerbell style', i.e. only taking instructions from the \
+              ringing-room calls. This is equivalent to using the '-us' flags."
+    )
 
-    args = parser.parse_args()
+    # Rhythm arguments
+    rhythm_group = parser.add_argument_group("Rhythm arguments")
+    rhythm_group.add_argument(
+        "-k", "--keep-going",
+        action="store_true",
+        help="If set, Wheatley will not wait for users to ring - instead, he will push on with the \
+              rhythm."
+    )
+    rhythm_group.add_argument(
+        "-w", "--wait",
+        action="store_true",
+        help="Legacy parameter, which is now set by default. The previous default behaviour of not waiting \
+              can be set with '-k'/'--keep-going'."
+    )
+    rhythm_group.add_argument(
+        "-I", "--inertia",
+        type=float,
+        default=0.5,
+        help="Overrides Wheatley's 'inertia' - now much Wheatley will take other ringers' positions \
+              into account when deciding when to ring.  0.0 means he will cling as closely as \
+              possible to the current rhythm, 1.0 means that he will completely ignore the other \
+              ringers."
+    )
+    rhythm_group.add_argument(
+        "-S", "--peal-speed",
+        default="2h58",
+        help="Sets the default speed that Wheatley will ring (assuming a peal of 5040 changes), \
+              though this will usually be adjusted by Wheatley whilst ringing to keep with other \
+              ringers.  Example formatting: '3h4' = '3h4m' = '3h04m' = '3h04' = '184m' = '184'. \
+              Defaults to '2h58'."
+    )
+    rhythm_group.add_argument(
+        "-G", "--handstroke-gap",
+        type=float,
+        default=1.0,
+        help="Sets the handstroke gap as a factor of the space between two bells.  Defaults to \
+              '1.0'."
+    )
+    rhythm_group.add_argument(
+        "-X", "--max-bells-in-dataset",
+        type=int,
+        default=15,
+        help="Sets the maximum number of bells that Wheatley will store to determine the current \
+              ringing speed.  If you make this larger, then he will be more consistent but less \
+              quick to respond to changes in rhythm.  Setting both this and \
+              --inertia to a very small values could result in Wheatley ringing ridiculously \
+              quickly.  Defaults to '15'."
+    )
 
+    # Misc arguments
+    parser.add_argument(
+        "--version",
+        action="version",
+        version=f"Wheatley v{__version__}"
+    )
+
+    # Parse arguments
+    # `[1:]` is apparently needed, because sys.argv[0] is the working file of the Python interpreter
+    # which `parser.parse_args` does not want to see as an argument
+    args = parser.parse_args(sys.argv[1:] if override_args is None else override_args)
+
+    # Deprecation warnings
     if args.wait:
         print("Deprecation warning: `--wait` has been replaced with `--keep-going`!")
 
@@ -236,22 +269,25 @@ def main():
     configure_logging()
 
     try:
-        socket_url = get_load_balancing_url(args.room_id, args.url)
+        tower_url = get_load_balancing_url(args.room_id, args.url)
     except TowerNotFoundError as e:
         sys.exit(f"Bad value for 'room_id': {e}")
     except InvalidURLError as e:
         sys.exit(f"Bad value for '--url': {e}")
 
-    tower = RingingRoomTower(args.room_id, socket_url)
-    bot = Bot(tower, row_generator(args), args.use_up_down_in or args.handbell_style,
-              args.stop_at_rounds or args.handbell_style, rhythm=rhythm(args))
+    tower = RingingRoomTower(args.room_id, tower_url)
+    row_generator = create_row_generator(args)
+    rhythm = create_rhythm(args)
+    bot = Bot(tower, row_generator, args.use_up_down_in or args.handbell_style,
+              args.stop_at_rounds or args.handbell_style, rhythm, user_name=args.name)
 
     with tower:
         tower.wait_loaded()
 
         print("=== LOADED ===")
 
-        bot.main_loop()
+        if not stop_on_join_tower:
+            bot.main_loop()
 
 
 if __name__ == "__main__":
