@@ -13,6 +13,7 @@ from wheatley.bell import Bell
 from wheatley.regression import calculate_regression
 from wheatley.tower import HANDSTROKE, BACKSTROKE, stroke_to_string
 
+
 WEIGHT_REJECTION_THRESHOLD = 0.001
 
 
@@ -21,7 +22,6 @@ def inverse_lerp(a, b, c):
     Inverse function to `lerp`.  Calculates t such that lerp(a, b, t) = c.
     (will divide by zero if a = b)
     """
-
     return (c - a) / (b - a)
 
 
@@ -30,7 +30,6 @@ def lerp(a, b, t):
     Linearly interpolates (unclamped) between a and b with factor t.
     Acts such that `lerp(a, b, 0.0)` returns `a`, and `lerp(a, b, 1.0)` returns `b`.
     """
-
     return (1 - t) * a + t * b
 
 
@@ -44,8 +43,6 @@ class Rhythm(metaclass=ABCMeta):
                            user_controlled: bool, stroke: bool):
         """ Sleeps the thread until a given Bell should have rung. """
 
-        pass
-
     @abstractmethod
     def expect_bell(self, expected_bell: Bell, row_number: int, place: int, expected_stroke: bool):
         """
@@ -55,24 +52,18 @@ class Rhythm(metaclass=ABCMeta):
         ringing.
         """
 
-        pass
-
     @abstractmethod
     def on_bell_ring(self, bell: Bell, stroke: bool, real_time: float):
         """
         Called when a bell is rung at a given stroke.  Used as a callback from the Tower class.
         """
 
-        pass
-
     @abstractmethod
     def initialise_line(self, stage: int, user_controls_treble: bool, start_time: float,
                         number_of_user_controlled_bells: int):
         """ Allow the Rhythm object to initialise itself when 'Look to' is called. """
 
-        pass
-
-    def sleep(self, seconds: float):
+    def sleep(self, seconds: float): #  pylint: disable=no-self-use
         """ Sleeps for given number of seconds. Allows mocking in tests"""
         time.sleep(seconds)
 
@@ -88,18 +79,19 @@ class WaitForUserRhythm(Rhythm):
         Initialise a wrapper around another Rhythm class that will decorate that class with the
         ability to wait for other people to ring.
         """
-
         self._inner_rhythm = rhythm
+
         self._current_stroke = HANDSTROKE
+
         self._expected_bells = {HANDSTROKE: set(), BACKSTROKE: set()}
         self._early_bells = {HANDSTROKE: set(), BACKSTROKE: set()}
+
         self.delay = 0
+
         self.logger = logging.getLogger(self.logger_name)
 
     def wait_for_bell_time(self, current_time, bell, row_number, place, user_controlled, stroke):
         """ Sleeps the thread until a given Bell should have rung. """
-
-        # assert stroke == self._current_stroke
         if stroke != self._current_stroke:
             self.logger.debug(f"Switching to unexpected stroke {stroke_to_string(stroke)}")
             self._current_stroke = stroke
@@ -112,9 +104,10 @@ class WaitForUserRhythm(Rhythm):
                 self.sleep(self.sleep_time)
                 delay_for_user += self.sleep_time
                 self.logger.debug(f"Waiting for {bell}")
+
             if delay_for_user:
-                self.logger.info(f"Delayed for {delay_for_user}")
                 self.delay += delay_for_user
+                self.logger.info(f"Delayed for {delay_for_user}")
 
     def expect_bell(self, expected_bell, row_number, place, expected_stroke):
         """
@@ -123,7 +116,6 @@ class WaitForUserRhythm(Rhythm):
         _should_ have been in the ringing, and so can use that knowledge to inform the speed of the
         ringing.
         """
-
         self._inner_rhythm.expect_bell(expected_bell, row_number, place, expected_stroke)
 
         if expected_stroke != self._current_stroke:
@@ -161,12 +153,14 @@ class WaitForUserRhythm(Rhythm):
     def initialise_line(self, stage, user_controls_treble, start_time,
                         number_of_user_controlled_bells):
         """ Allow the Rhythm object to initialise itself when 'Look to' is called. """
-
         self._expected_bells[HANDSTROKE].clear()
         self._expected_bells[BACKSTROKE].clear()
+
         self._early_bells[HANDSTROKE].clear()
         self._early_bells[BACKSTROKE].clear()
+
         self._current_stroke = HANDSTROKE
+
         # Clear any current waiting loops
         self.sleep(2 * self.sleep_time)
 
@@ -181,13 +175,13 @@ class RegressionRhythm(Rhythm):
 
     logger_name = "RHYTHM:Regression"
 
-    def __init__(self, inertia, peal_speed=178, handstroke_gap=1, max_rows_in_dataset=3.0):
+    def __init__(self, inertia, peal_speed=178, handstroke_gap=1, min_bells_in_dataset=4,
+                 max_bells_in_dataset=15):
         """
         Initialises a new RegressionRhythm with a given default peal speed and handstroke gap.
         `peal_speed` is the number of minutes in a peal of 5040 changes, and `handstroke_gap`
         measures how many places long the handstroke gap is.
         """
-
         # An inertia-like coefficient designed to allow the regression finder to slowly adjust to
         # a new rhythm
         # 0.0 means that a new regression line will take effect instantly
@@ -196,19 +190,19 @@ class RegressionRhythm(Rhythm):
 
         self._peal_speed = peal_speed
         self._handstroke_gap = handstroke_gap
-        self._max_rows_in_dataset = max_rows_in_dataset
+        self._min_bells_in_dataset = min_bells_in_dataset
+        self._max_bells_in_dataset = max_bells_in_dataset
 
         self.stage = 0
-        self.logger = logging.getLogger(self.logger_name)
-
         self._start_time = 0
         self._blow_interval = 0
 
         self._number_of_user_controlled_bells = 0
-
         self._expected_bells = {}
         self.data_set = []
 
+        self.logger = logging.getLogger(self.logger_name)
+
     def _add_data_point(self, row_number, place, real_time, weight):
         blow_time = self.index_to_blow_time(row_number, place)
 
@@ -217,11 +211,9 @@ class RegressionRhythm(Rhythm):
         for (b, r, w) in self.data_set:
             self.logger.debug(f"  {b}\t{r}\t{w}")
 
-        max_dataset_length = self._max_rows_in_dataset * self._number_of_user_controlled_bells
-
         # Only calculate the regression line if there are at least two datapoints, otherwise
         # just store the datapoint
-        if len(self.data_set) >= 2:
+        if len(self.data_set) >= self._min_bells_in_dataset:
             (new_start_time, new_blow_interval) = calculate_regression(self.data_set)
 
             # Lerp between the new times and the old times, according to the desired inertia
@@ -239,17 +231,19 @@ class RegressionRhythm(Rhythm):
             self.data_set = list(filter(lambda d: d[2] > WEIGHT_REJECTION_THRESHOLD, self.data_set))
 
             # Eventually forget about datapoints
-            if len(self.data_set) >= max_dataset_length:
+            if len(self.data_set) >= self._max_bells_in_dataset:
                 del self.data_set[0]
 
     def wait_for_bell_time(self, current_time, bell, row_number, place, user_controlled, stroke):
         """ Sleeps the thread until a given Bell should have rung. """
-
         if user_controlled and self._start_time == float('inf'):
-            self.logger.debug(f"Waiting for pull off")
+            self.logger.debug("Waiting for pull off")
+
             while self._start_time == float('inf'):
                 self.sleep(0.01)
-            self.logger.debug(f"Pulled off")
+
+            self.logger.debug("Pulled off")
+
             return
 
         bell_time = self.index_to_real_time(row_number, place)
@@ -269,7 +263,6 @@ class RegressionRhythm(Rhythm):
         _should_ have been in the ringing, and so can use that knowledge to inform the speed of the
         ringing.
         """
-
         self.logger.debug(f"Expected bell {expected_bell} at index {row_number}:{place} at stroke"
                           + f"{expected_stroke}")
         self._expected_bells[(expected_bell, expected_stroke)] = (row_number, place)
@@ -278,14 +271,11 @@ class RegressionRhythm(Rhythm):
         """
         Called when a bell is rung at a given stroke.  Used as a callback from the Tower class.
         """
-
         # If this bell was expected at this stroke (i.e. is being rung by someone else)
         if (bell, stroke) in self._expected_bells:
             # Figure out where the bell was expected in ringing space
             (row_number, place) = self._expected_bells[(bell, stroke)]
-
             expected_blow_time = self.index_to_blow_time(row_number, place)
-
             diff = self.real_time_to_blow_time(real_time) - expected_blow_time
 
             self.logger.info(f"{bell} off by {diff} places")
@@ -317,7 +307,6 @@ class RegressionRhythm(Rhythm):
     def initialise_line(self, stage, user_controls_treble, start_time,
                         number_of_user_controlled_bells):
         """ Allow the Rhythm object to initialise itself when 'Look to' is called. """
-
         self._number_of_user_controlled_bells = number_of_user_controlled_bells
         self.stage = stage
 
@@ -343,12 +332,10 @@ class RegressionRhythm(Rhythm):
     # Linear conversions between different time measurements
     def index_to_blow_time(self, row_number, place):
         """ Convert a row number and place into a blow_time, taking hanstroke gaps into account. """
-
         return row_number * self.stage + place + (row_number // 2) * self._handstroke_gap
 
     def blow_time_to_real_time(self, blow_time):
         """ Convert from blow_time into real_time using the regression line. """
-
         return self._start_time + self._blow_interval * blow_time
 
     def index_to_real_time(self, row_number, place):
@@ -356,10 +343,8 @@ class RegressionRhythm(Rhythm):
         Convert straight from row number and place into the expected real time according to the
         regression line.
         """
-
         return self.blow_time_to_real_time(self.index_to_blow_time(row_number, place))
 
     def real_time_to_blow_time(self, real_time):
         """ Convert backwards from a real time to the corresponding blow time. """
-
         return (real_time - self._start_time) / self._blow_interval
